
# q-expansion of newform 186.2.m.b, downloaded from the LMFDB on 28 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 186
weight = 2
poly_data = [1, -1, 0, 1, -1, 1, 0, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[125, [1, 0, 0, 0, 0, 0, 0, 0]], [127, [-1, 1, 0, -1, 1, -1, 0, 1]]]
aps_data = [[0, 0, 0, 0, 0, 0, -1, 0], [0, 0, 0, 0, -1, 0, 0, 0], [0, 0, 0, 0, 1, -2, 1, 0], [2, 1, -2, 1, 0, -2, 3, -1], [-3, 2, 0, 0, 1, -2, -2, 2], [-1, 0, 2, -3, 2, 1, -3, 1], [0, -1, 2, -1, -1, 3, 1, 0], [3, -3, 0, 2, -1, 1, 1, -2], [0, 0, -1, 4, -1, 0, 0, 0], [-5, 0, 5, 0, 0, -1, 0, 4], [5, -2, -1, 1, -2, 4, 3, -7], [1, -1, 2, -4, 2, 4, -6, 3], [1, 0, 3, -3, -3, 3, 0, 1], [6, -3, -4, 4, -2, 6, 2, -8], [4, -6, -2, 0, -4, 8, 0, -4], [-8, 3, -1, -6, 7, -5, -1, 5], [0, -5, 1, 0, -2, 0, 1, -5], [2, -1, 0, -1, 3, -1, -2, 2], [-2, 2, 2, 3, -2, -8, -4, 7], [5, -8, -2, 3, -1, 10, -7, -5], [-12, 2, 2, -3, -2, -5, -5, 5], [-7, 5, 4, -2, 2, -2, 1, 4], [-2, -3, -3, -2, -2, -3, -3, -2], [-4, 8, -5, 5, 2, -6, 9, -1], [0, 0, 7, -7, -3, -3, -7, 7], [2, -3, -4, 1, -4, -3, 2, 0], [0, -10, 9, -2, -2, 9, -10, 0], [8, -10, 4, -2, -3, 5, -7, -1], [-5, -2, -8, -2, 3, -6, 3, 0], [-2, 9, 4, 3, 10, -6, -6, 6], [0, 9, 1, -6, 1, -6, 1, 9], [3, -6, -4, 10, 1, -5, -2, -10], [5, 5, -5, -5, 0, 5, 5, 0], [-11, 2, -1, -4, 7, -14, -4, 11], [18, -10, 9, 15, -13, 15, 6, -3], [-3, 6, -4, 15, 1, -5, 18, -1], [7, -7, 0, 7, -14, 10, 3, -4], [-2, 1, -1, -6, -1, 1, -2, 0], [9, 8, -4, 10, -5, 3, 5, -8], [-8, 16, -7, -10, 9, 1, 7, 9], [1, 4, 0, -4, -1, 1, 4, 4], [-12, 12, 12, -15, 13, -5, 1, 9], [7, -7, -7, -2, 1, 7, 8, -16], [3, -9, -1, 10, -2, 1, -8, 0], [-6, 2, 13, -3, 4, -5, -5, 5], [-10, -3, 7, 2, 4, -6, 1, 12], [5, 9, 5, -1, 12, 8, -6, 3], [7, 0, -13, 5, -9, -2, 18, -9], [-3, -6, -7, -3, -3, -7, -6, -3], [23, -20, -6, 20, -10, 9, 10, -19], [6, 3, -3, 6, 0, 0, 6, -6], [-10, -6, 2, 3, -5, -7, 10, -1], [13, -6, -2, 8, -11, 9, -4, 0], [0, 5, -2, -17, 5, -17, -2, 5], [11, 8, -4, 5, -1, -2, 7, -9], [-18, 5, 10, -14, 4, -8, -9, 13], [-2, -15, 1, -5, -5, 1, -15, -2], [-13, 1, 12, -6, 12, 1, -13, 0], [3, 0, -3, 0, 0, -3, -15, -6], [9, -1, -8, 1, -2, 11, -11, 1], [-4, 8, -1, -4, -8, -16, 0, 3], [-9, 5, 8, -4, 10, -11, -10, 14], [0, -3, 1, -10, -11, -10, 1, -3], [1, 14, -3, 3, -14, 0, 14, -3], [0, -2, -9, 3, -1, 3, -9, -2], [-2, 2, 2, -4, 0, 2, 0, -11], [2, 0, -1, 10, 10, -1, 0, 2], [3, -6, -1, 18, -3, 3, 15, -4], [-20, 3, 11, -15, 16, -7, -26, 13], [-6, -4, -10, 8, -10, -4, -6, 0], [14, -5, -9, 16, -8, 7, 8, -15], [-23, 9, 8, -11, 17, -6, -6, 19], [-1, 1, 1, -7, 0, 18, -1, -5], [6, -1, -9, 14, -9, 5, 4, -19], [-16, 11, 10, -21, 6, 4, 1, 21], [19, -2, -4, 9, -3, 11, 11, -11], [-15, 10, 10, -5, 23, -12, -7, 28], [27, -1, 1, 5, -3, 25, 4, -2], [-14, -2, 16, 2, -4, -11, 0, 1], [-1, 12, -17, 1, 3, -10, 18, -9], [24, -3, -18, 19, -5, 10, 20, -25], [-2, -2, 12, -18, 10, 8, -12, 10], [0, -13, -3, 11, -17, 11, -3, -13], [-4, 7, 9, -9, -7, 0, 7, 9], [8, -8, -8, 12, -8, 18, 0, -4], [9, -1, -1, 9, -8, 1, 8, -10], [33, -9, -10, 20, -13, 26, 14, -27], [-23, -6, -3, 1, -3, -6, -23, 0], [-6, 12, -5, 7, -12, -24, 13, 1], [-2, 1, 1, -1, 2, -5, -8, -2], [7, 0, -7, 0, 0, 2, 17, -5], [-11, 16, -10, 5, 19, -19, 2, 14], [20, -12, 8, 12, -20, 20, -8, -4], [8, -8, -8, 7, -4, 9, 4, -9], [0, -23, -5, 5, -18, 5, -5, -23], [1, -5, -2, 7, 1, -3, -3, -10], [-6, 3, 7, -14, 10, 4, -15, 2], [2, 11, -14, 2, 3, -6, 16, -8], [-13, 26, 15, -8, 11, -15, 5, 28], [5, -3, -4, 2, 6, 12, 11, 4], [-2, -4, -9, -5, 0, -1, -1, 1], [-19, 19, 4, -13, -1, -9, 10, 22], [2, -2, -2, 5, -1, -8, 1, 1], [15, -1, -9, -1, 15, 0, 0, 0], [3, -22, 11, 19, -11, -8, -7, -7], [-29, 3, 24, -24, 12, -29, -12, 41], [13, -9, -4, 9, -18, -1, 21, -23], [-15, 30, 0, -20, 24, -6, -5, 15], [-7, -2, 4, -19, -19, 4, -2, -7], [-16, -8, 6, -20, 10, -2, -10, 12], [14, 10, -19, 4, -9, 5, 16, -12], [-1, -7, -4, 11, 5, -9, -3, -27], [0, 7, -2, -2, 23, -2, -2, 7], [1, 9, -10, 14, -17, 4, 13, -18], [29, -9, -11, 2, 7, 11, 11, -11], [-11, 6, -15, -6, -6, -15, 6, -11], [-14, -2, 2, 7, 2, -2, -14, 0], [15, -30, -5, 9, -9, 21, -6, -20], [-9, 5, 4, -5, 10, -8, -4, 6], [17, 28, -8, 7, -8, 28, 17, 0], [9, 0, 24, 4, 4, 24, 0, 9], [8, 31, -3, 2, 1, -25, 27, -5], [4, -4, -4, 0, -8, 9, -4, -8], [0, -1, -24, 20, 9, -4, -5, -16], [20, -15, -4, 9, -5, 26, -17, -13], [-16, -9, -9, -6, 3, -1, -3, 4], [-8, 16, -13, 4, 16, 0, 12, -5], [33, -16, 12, 18, -19, 30, 6, -3], [6, -4, -4, 2, -16, 9, 7, -18], [-10, -3, -5, 6, -28, 9, 9, -9], [13, -13, -13, 19, -3, -12, 10, -7], [3, -23, 10, -3, 9, 7, -16, -4], [-14, 1, -7, 6, 21, -28, 8, 3], [-1, 8, -8, 1, 0, -1, 4, -4], [26, -16, -20, 10, -19, 22, 16, -29], [-9, 9, 2, -10, 15, -3, -12, 6], [-1, 6, -5, -6, 12, 9, -11, 16], [4, 9, -13, 16, -8, 1, 8, -9], [-30, -1, 9, -29, 20, 0, -29, 38], [0, 15, 1, 5, -5, 5, 1, 15], [-7, 7, 7, -19, 8, 18, 1, -5], [-15, 17, 15, -13, 2, -4, 0, 2], [53, -45, -16, 8, -25, 26, -11, -33], [-6, -10, 0, 15, 15, 0, -10, -6], [-3, -1, 12, -26, 12, -1, -3, 0], [-3, -2, 5, 2, -4, -5, -5, -4], [18, -30, 3, 6, -12, 24, -21, 9], [-31, 3, 30, -22, 19, -25, -25, 25], [31, 10, -1, 9, -8, 12, -3, -10], [-6, 6, 6, -16, -2, -32, -8, -4], [-8, -5, -5, 5, 0, -8, 13, -10], [46, -19, 0, -8, -19, 27, -6, -25], [13, -5, -19, 6, -7, 14, 21, -28], [-45, 24, 13, -20, 10, -38, -10, 48], [1, -12, 3, 11, 3, -12, 1, 0], [5, 22, 3, 14, -7, -12, 7, 5], [-9, -7, 20, -11, -1, -4, -4, 4], [29, 6, -9, 20, -28, 11, 17, -31], [-34, 19, 0, -19, 34, -34, 19, 30], [-6, -20, -5, -15, -16, 5, 5, -5], [7, -16, 18, -9, 12, 12, -13, 21], [11, -9, -11, -16, -11, -9, 11, 0], [9, -33, -4, -12, -29, 13, -8, 4], [14, 12, -20, 11, 11, -20, 12, 14], [6, 15, -21, 9, 3, -6, 33, -30], [30, 20, -15, 9, 6, 1, 8, -24], [39, -2, -14, 24, -18, 10, 8, -34], [-12, 12, 12, 18, -1, -22, -13, 42]]
